/*******************************************************************
 * 
 * Wifiradio
 * Andreas Brietzke, Nagareyama, J
 *
 * Chip type           : ATmega88
 * Clock frequency     : 3,6864 MHz
 *
 *     wifiradio.c
 * 
 *     key routines from Peter Dannegger
 *
 ********************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <string.h>
#include <stdio.h>
#include <inttypes.h>
#include <stdlib.h>

#include "lcd.h"
#include "usart.h"		// own usart lib

#define KEY_PIN		PIND
#define KEYMENU		2
#define KEYSTOR		3
#define KEYDOWN		4
#define KEYUP		5

#define REPEAT_MASK	(1<<KEYMENU^1<<KEYSTOR^1<<KEYDOWN^1<<KEYUP)
#define REPEAT_START	50		  // after 500ms
#define REPEAT_NEXT	20		  // every 200ms

#define LCD_DISP_LENGTH 20
#define LCD_SCROLL_DELAY 3

uint8_t key_state;			// debounced and inverted key state: bit = 1: key pressed
uint8_t key_press;			// key press detect
uint8_t key_rpt;			// key long press and repeat

uint8_t station_no = 1;
uint8_t station_max = 1;
uint8_t old_station_no = 0;

uint8_t Volume = 1;
uint8_t old_Volume = 0;

uint8_t fStarted = 0;

uint8_t fDisplay;
uint8_t iDisplay;
uint8_t iDisplayDelayTitle;
uint8_t iDisplayDelayName;

uint8_t fSendRouter;
uint8_t iSendRouter;

uint8_t SetVolume[11]= {0,8,9,10,12,15,20,30,50,70,100};


ISR(TIMER0_OVF_vect) {			// every 10ms
  static uint8_t ct0, ct1, rpt;
  uint8_t i;
  
  TCNT0 = (uint8_t)(int16_t)-(F_CPU / 1024 * 10e-3 + 0.5);	// preload for 10ms
  
  i = key_state ^ ~KEY_PIN;		// key changed ?
  ct0 = ~( ct0 & i );			// reset or count ct0
  ct1 = ct0 ^ (ct1 & i);		// reset or count ct1
  i &= ct0 & ct1;			// count until roll over ?
  key_state ^= i;			// then toggle debounced state
  key_press |= key_state & i;		// 0->1: key press detect
  
  if( (key_state & REPEAT_MASK) == 0 )	// check repeat function
     rpt = REPEAT_START;		// start delay
     if( --rpt == 0 ){
       rpt = REPEAT_NEXT;			// repeat delay
       key_rpt |= key_state & REPEAT_MASK;
     }
     

  iSendRouter++;
  if (iSendRouter==10) {
    iSendRouter=0;
    fSendRouter=1;       // send infos to router (new station ect)
  }
  
  iDisplay++;
  if (iDisplay== 30) {
    iDisplay=0;
    fDisplay=1;      // refresh display all 100ms
  }
  
}

uint8_t get_key_press( uint8_t key_mask ) {
  cli();					// read and clear atomic !
  key_mask &= key_press;                        // read key(s)
  key_press ^= key_mask;                        // clear key(s)
  sei();
  return key_mask;
}

uint8_t get_key_rpt( uint8_t key_mask ) {
  cli();					// read and clear atomic !
  key_mask &= key_rpt;                        	// read key(s)
  key_rpt ^= key_mask;                        	// clear key(s)
  sei();
  return key_mask;
}

uint8_t get_key_short( uint8_t key_mask ) {
  cli();			// read key state and key press atomic !
  return get_key_press( ~key_state & key_mask );
}

uint8_t get_key_long( uint8_t key_mask ) {
  return get_key_press( get_key_rpt( key_mask ));
}

void refresh_display (char *sTitle, char *sName) {
  char sOutString[20];
  static uint8_t iCountName=0;
  static uint8_t iCountTitle=0;
  
  char sLCD[LCD_DISP_LENGTH+1];
  char *strLCD;

  uint8_t iStrLen;
  uint8_t iDiffLen;
  uint8_t i;

  strLCD=sLCD;
  strLCD = strcpy(sLCD,"Vol:");
  strLCD = strcat(sLCD,ltoa( Volume, sOutString, 10 ));
  strLCD = strcat(sLCD,"/10   ");
  lcd_gotoxy(0,3);
  lcd_puts(strLCD);
  
  strLCD = strcpy(sLCD,"Sta:");
  strLCD = strcat(sLCD,ltoa( station_no, sOutString, 10 ));
  strLCD = strcat(sLCD,"/");
  strLCD = strcat(sLCD, ltoa( station_max, sOutString, 10 ));
  strLCD = strcat(sLCD,"  ");
  lcd_gotoxy(11,3);
  lcd_puts(strLCD);   
    
  iStrLen=strlen(sName);
  if (iStrLen <= LCD_DISP_LENGTH) {                     // fill with spaces
    iDiffLen=LCD_DISP_LENGTH-iStrLen;
    for (i=1; i<=iDiffLen; i++) {
      strcat(sName," ");  
    }
    lcd_gotoxy(0,0);
    lcd_puts(sName);
  } else {                                              // roll string
    iDiffLen= iStrLen - LCD_DISP_LENGTH;
    if (iCountName++ >= iDiffLen) {
      iCountName=0;
      iDisplayDelayName=0;
    }
    if (iDisplayDelayName++ < LCD_SCROLL_DELAY) iCountName=0;
    strncpy(sLCD,sName+iCountName,LCD_DISP_LENGTH);
    sLCD[LCD_DISP_LENGTH]='\0';
    lcd_gotoxy(0,0);
    lcd_puts(sLCD);    
  }
  
  iStrLen=strlen(sTitle);
  if (iStrLen <= LCD_DISP_LENGTH) {                     // fill with space
    iDiffLen=LCD_DISP_LENGTH-iStrLen;
    for (i=1; i<=iDiffLen; i++) {
      strcat(sTitle," ");  
    }
    lcd_gotoxy(0,1);
    lcd_puts(sTitle);
  } else {                                              // roll string
    iDiffLen= iStrLen - LCD_DISP_LENGTH;
    if (iCountTitle++ >= iDiffLen) {
      iCountTitle=0;
      iDisplayDelayTitle=0;
    }
    if (iDisplayDelayTitle++ < LCD_SCROLL_DELAY) iCountTitle=0;
    strncpy(sLCD,sTitle+iCountTitle,LCD_DISP_LENGTH);
    sLCD[LCD_DISP_LENGTH]=0x00;
    lcd_gotoxy(0,1);
    lcd_puts(sLCD);
  }
  
  fDisplay=0;
}

void check_keys (void) {
  if(get_key_press(1<<KEYMENU)) {
    if (Volume<10) Volume = Volume++;
  }
  
  if(get_key_press(1<<KEYSTOR)) {
    if (Volume>0) Volume = Volume--;
  }
  
  if(get_key_press(1<<KEYDOWN)) {
    station_no--;
    if (station_no==0) station_no=station_max;
  }
  
  if(get_key_press(1<<KEYUP)) {
    station_no++;
    if (station_no>station_max) station_no=1;
  } 
}

void send_router (void) {
  char sOutString[20];
  // change the station
  if (station_no != old_station_no) {       
    UsartPuts("cmd_station: ");
    UsartPuts(ltoa(station_no, sOutString, 10));
    UsartPuts("\n");
    old_station_no = station_no;
  }
  // change the volume
  if (Volume != old_Volume) {       
    UsartPuts("cmd_volume: ");
    UsartPuts(ltoa(SetVolume[Volume], sOutString, 10));
    UsartPuts("\n");
    old_Volume = Volume;
  }
  fSendRouter=0;
}


int main( void ) {
  char sCommandString[BUFFER_LENGHT];
  char sCommandPar[BUFFER_LENGHT];
  
  char sTitle[BUFFER_LENGHT];
  char sName[BUFFER_LENGHT];
  
  // Key inputs
  DDRD &= ~( (1<<KEYMENU) | (1<<KEYSTOR) | (1<<KEYDOWN) | (1<<KEYUP) );
  // activate pullup resistors for keys
  PORTD |= ((1<<KEYMENU) | (1<<KEYSTOR) | (1<<KEYDOWN) | (1<<KEYUP));
  
  // init timer  
  TCCR0B = 1<<CS02^1<<CS00;			// divide by 1024
  TIMSK0 = 1<<TOIE0;				// enable timer overflow interrupt
  
  sei();
  InitUsart();
  lcd_init(LCD_DISP_ON_CURSOR);
  
  lcd_clrscr();
  lcd_puts("Andys Webradio V 1.0");
  lcd_gotoxy(0,2);
  lcd_puts("booting......");
  lcd_gotoxy(0,3);
  lcd_puts("please wait!");
  
  while (1) {
    if (GetCommand(sCommandPar, sCommandString) !=0) {      
      
      if(strcmp(sCommandString, "Stations:") == 0) {
        station_max = atoi(sCommandPar);   
        lcd_clrscr();
        fStarted=1;    
        UsartPuts("\n");
        UsartPuts("tuner: 1\n");
      }
      
      if(strcmp(sCommandString, "Name:") == 0) {
        strcpy(sName, sCommandPar);       
      }
      
      if(strcmp(sCommandString, "Title:") == 0) {
        strcpy(sTitle, sCommandPar);
      }
      
      ResetCommand();
    }   
    
    if (fStarted)  {  
      
      if (fDisplay) {
        refresh_display(sTitle, sName);
      }
      
      if (fSendRouter) {
        send_router();    
      }
      
      check_keys();
    }
  }
  
  
}
// key examples

// 			// single press
//     if( get_key_press( 1<<KEYMENU ))

// 			// release after short press: task 1
//     if( get_key_short( 1<<KEYSTOR ))

// 			// long press: task 2
//     if( get_key_long( 1<<KEYSTOR ))

// 			// single press and repeat
//     if( get_key_press( 1<<KEYDOWN ) || get_key_rpt( 1<<KEYDOWN )){
  